/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package com.company.catweb.ejb;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import com.company.catweb.model.*;
import java.rmi.AlreadyBoundException;
import java.util.List;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

/**
 *
 * @author Wim
 */
@Stateless
@Local(CatWebSessionLocal.class)
@Remote(CatWebSessionRemote.class)
public class CatWebSessionBean implements CatWebSessionRemote, CatWebSessionLocal {
    
    @PersistenceContext
    private EntityManager em;

    public Catalog readCatalog(int id) {
        Catalog c=null;
        try {
            c = (Catalog) em.find(Catalog.class, id);
        }
        catch (Exception e) {
            System.err.println("error occurred while fetching catalog....");
        }
        return c;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean createCatalog(Catalog c) {
    try {
            Catalog catalog = (Catalog) em.find(Catalog.class, c.getName());

            if (catalog!=null) {
                System.out.println("add: found an existing catalog obj for :" + c.getName());
                throw new AlreadyBoundException();
            }
            else persist(c);
        } catch (Exception e) {
            System.err.println("error occurred while creating catalog....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean updateCatalog(Catalog c) throws Exception{
        try {
            persist(c);
        } catch (Exception e) {
            System.err.println("unknown error occurred while updating catalog....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean deleteCatalog(int id) {
        Catalog c = null;
        try {
            c=this.readCatalog(id);
            remove(c);
        } catch (Exception e) {
            System.err.println("unknown error occurred while deleting store.....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public Category readCategory(int id) {
        Category c=null;
        try {
            c = (Category) em.find(Category.class, id);
        }
        catch (Exception e) {
            System.err.println("error occurred while fetching catalog....");
        }
        return c;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean createCategory(Category c) {
    try {
            Category category = (Category) em.find(Category.class, c.getName());

            if (category!=null) {
                System.out.println("add: found an existing catalog obj for :" + c.getName());
                throw new AlreadyBoundException();
            }
            else persist(c);
        } catch (Exception e) {
            System.err.println("error occurred while creating catalog....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean updateCategory(Category c) {
        try {
            persist(c);
        } catch (Exception e) {
            System.err.println("unknown error occurred while updating catalog....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean deleteCategory(int id) {
        try {
            Category category = (Category) em.find(Category.class, id);
            Catalog catalog = (Catalog) em.find(Catalog.class, category.getCatalogId());

            catalog.getCategoriesCollection().remove(category);
        } catch (Exception e) {
            System.err.println("error occurred while removing category from catalog.....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public Product readProduct(int id) {
        Product c=null;
        try {
            c = (Product) em.find(Product.class, id);
        }
        catch (Exception e) {
            System.err.println("error occurred while fetching product....");
        }
        return c;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean createProduct(Product c) {
    try {
            Category category = (Category) em.find(Category.class, c.getCategoryId());

            if (category!=null) {
                System.out.println("add: found an existing product in calatlog obj for :" + c.getName());
                throw new AlreadyBoundException();
            }
            else persist(c);
        } catch (Exception e) {
            System.err.println("error occurred while creating product....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean updateProduct(Product c) {
        try {
            persist(c);
        } catch (Exception e) {
            System.err.println("unknown error occurred while updating product....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    public boolean deleteProduct(int id) {
        try {
            Product product = (Product) em.find(Product.class, id);
            Category cat = (Category) em.find(Category.class, product.getCategoryId());

            cat.getProductsCollection().remove(product);
        } catch (Exception e) {
            System.err.println("error occurred while removing product from catalog.....");
            e.printStackTrace();
            return false;
        }
        return true;
    }

    public List findCatalogs(String n) {
        List dps = em.createNamedQuery("Catalog.findByName").setParameter("name", n).getResultList();
        return dps;
    }

    public List findCategories(String n) {
        List dps = em.createNamedQuery("Category.findByName").setParameter("name", n).getResultList();
        return dps;
    }

    public List findProducts(String n) {
        List dps = em.createNamedQuery("Product.findByName").setParameter("name", n).getResultList();
        return dps;
    }


    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    private void remove(Object obj) {
        Object mergedObj = em.merge(obj);
        em.remove(mergedObj);
    }

    @TransactionAttribute(TransactionAttributeType.REQUIRED)
    private void persist(Object obj) {
        em.persist(obj);
    }

}
